<?php

namespace App\Console\Commands;

use Illuminate\Console\Command;
use Illuminate\Support\Facades\Config;

use App\Models\{Service,Site};

class ServiceList extends Command
{
	/**
	 * The name and signature of the console command.
	 *
	 * @var string
	 */
	protected $signature = 'service:list'.
		' {--i|inactive : Include Inactive}'.
		' {--t|type= : Type}'.
		' {--f|fix : Fix start_date}';

	/**
	 * The console command description.
	 *
	 * @var string
	 */
	protected $description = 'List all services';

	/**
	 * Execute the console command.
	 *
	 * @return mixed
	 */
	public function handle()
	{
		$header = '|%13s|%-10s|%-35s|%-40s|%8s|%10s|%15s|%10s|%10s|%12s|%14s|';

		$this->warn(sprintf($header,
			'ID',
			'Type',
			'Product',
			'Name',
			'active',
			'status',
			'invoice next',
			'start date',
			'stop date',
			'connect date',
			'first invoice'
		));

		foreach (Service::withoutGlobalScope(\App\Models\Scopes\SiteScope::class)->with(['site'])->cursor() as $o) {
			if ((! $this->option('inactive')) AND ! $o->isActive())
				continue;

			Config::set('site',$o->site);

			if ($this->option('type') AND ($o->product->getProductTypeAttribute() !== $this->option('type')))
				continue;

			$c = $o->invoice_items->filter(function($item) {return $item->item_type === 0; })->sortby('date_start')->first();

			if ($this->option('fix') AND ! $o->date_start AND $c AND $c->date_start AND $o->type AND $o->type->service_connect_date AND $c->date_start->format('Y-m-d') == $o->type->service_connect_date->format('Y-m-d')) {
				$o->date_start = $o->type->service_connect_date;
				$o->save();
			}

			$this->info(sprintf($header,
				$o->sid,
				$o->product->getProductTypeAttribute(),
				$o->product->getNameAttribute(),
				$o->name_short,
				$o->active ? 'active' : 'inactive',
				$o->status,
				$o->invoice_next ? $o->invoice_next->format('Y-m-d') : NULL,
				$o->date_start ? $o->date_start->format('Y-m-d') : NULL,
				$o->date_end ? $o->date_end->format('Y-m-d') : NULL,
				($o->type AND $o->type->service_connect_date) ? $o->type->service_connect_date->format('Y-m-d') : NULL,
				($c && $c->date_start) ? $c->date_start->format('Y-m-d') : NULL,
			));
		}
	}
}